package dev.peerat.framework;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.function.Consumer;
import java.util.function.Supplier;

import dev.peerat.framework.Locker.Key;

public class Locker<V>{
	
	private Map<Key, BlockingQueue<V>> map;
	
	public Locker(){
		this.map = new HashMap<>();
	}
	
	public void init(Key key){
		this.map.put(key, new LinkedBlockingQueue<>());
	}
	
	public void remove(Key key){
		this.map.remove(key);
	}
	
	private BlockingQueue<V> get(Key key){
		return this.map.get(key);
	}
	
	public void setValue(V value){
		for(Entry<Key, BlockingQueue<V>> entry : this.map.entrySet()){
			entry.getValue().add(value);
			this.unlock(entry.getKey());
		}
	}
	
	public V getValue(Key key){
		BlockingQueue<V> queue = get(key);
		if(queue.isEmpty()) return null;
		return queue.poll();
	}

	public void lock(Key key) throws InterruptedException{
		BlockingQueue<V> queue = get(key);
		if(queue.isEmpty()){
			synchronized(queue){
				queue.wait();
			}
		}
	}
	
	public void unlock(Key key){
		BlockingQueue<V> queue = get(key);
		synchronized(queue){
			queue.notify();
		}
	}
	
	public void listen(Consumer<V> action){
		Key key = new Key();
		init(key);
		try {
			while(true){
				lock(key);
				V value = getValue(key);
				action.accept(value);
			}
		}catch(Exception e){}
		remove(key);
	}
	
	public void listen(Supplier<Boolean> condition,Consumer<V> action){
		Key key = new Key();
		init(key);
		try {
			while(condition.get()){
				lock(key);
				V value = getValue(key);
				action.accept(value);
			}
		}catch(Exception e){}
		remove(key);
	}
	
	public Thread listenAsync(Consumer<V> action){
		Thread thread = new Thread(new Runnable(){
			public void run(){
				listen(action);
			}
		});
		thread.start();
		return thread;
	}
	
	public Thread listenAsync(Supplier<Boolean> condition,Consumer<V> action){
		Thread thread = new Thread(new Runnable(){
			public void run(){
				listen(condition, action);
			}
		});
		thread.start();
		return thread;
	}
	
	public static class Key{ public Key(){} }
}
